
# Script to generate size-free residuals from 
#	morpohmetric data.



# Function to extract residuals
computeResiduals <- function(x, y, V){
	
	X <- cbind(x, rep(1, length(x)));
	Y <- matrix(y, ncol=1);
	b <- solve(t(X) %*% solve(V) %*% X) %*% t(X) %*% solve(V) %*% Y;
	resid <- y - X %*% b;
	return(resid);
}

library(ape);
v <- read.tree('skinktree183.txt');
vmat <- vcv.phylo(v);

xx <- read.csv('morphodata.csv', header=T);

# Replace zero-valued measurements with a minimum:
MIN <- 0.1;
xx[,1:8][xx[,1:8] < MIN] <- MIN;


# Log transform morphology:
logmorph <- log(xx[,1:8]);

logmorph <- cbind(logmorph, xx[,9:11]);

rownames(logmorph) <- logmorph$treename;
logmorph <- logmorph[rownames(vmat), ];

dd <- logmorph[, 9:11];
dd$svl <- logmorph$svl;

for (i in 2:8){
	dd <- cbind(dd, computeResiduals(dd$svl, logmorph[,i], vmat));
}

nm <- names(logmorph)[2:8];
 
nm <- paste(nm, '_glsresid', sep='');

colnames(dd)[5:ncol(dd)] <- nm;

write.table(dd, file = 'sphenoSizeFreeResiduals_XXX.txt', sep=',', row.names=F, quote=F);

################

#dd <- read.table('sphenoSizeFreeResiduals_XXX.txt', sep=',', header=T);
###########################

# Do principal components on size-free residuals

pc <- princomp(dd[, 5:11]);
 
shape1 <- pc$scores[, 1];
shape2 <- pc$scores[ ,2];

plot(shape1 ~ dd$svl);

df.svl <- data.frame(species=rownames(dd), svl=dd$svl);
df.shape1 <- data.frame(species=names(shape2), shape1=as.numeric(shape1));
df.shape2 <- data.frame(species= names(shape2), shape2 = as.numeric(shape2));

write.table(df.svl, col.names=F, row.names=F, sep='\t', quote=F, file = 'skinks_svl.txt');
write.table(df.shape1, col.names=F, row.names=F, sep='\t', quote=F, file = 'skinks_shape1.txt');
write.table(df.shape2, col.names=F, row.names=F, sep='\t', quote=F, file = 'skinks_shape2.txt');

# View variance summary:
summary(pc);

# View loadings:
pc$loadings[,1]

#############
